@@ -516,6 +516,19 @@ gcal_edit_dialog_action_button_clicked (GtkWidget *widget,
       start_date = gcal_edit_dialog_get_date_start (dialog);
       end_date = gcal_edit_dialog_get_date_end (dialog);
 
+      /*
+       * The end date for multi-day events is exclusive, so we bump it by a day.
+       * This fixes the discrepancy between the end day of the event and how it
+       * is displayed in the month view. See bug 769300.
+       */
+      if (all_day)
+        {
+          GDateTime *fake_end_date = g_date_time_add_days (end_date, 1);
+
+          g_clear_pointer (&end_date, g_date_time_unref);
+          end_date = fake_end_date;
+        }
+
       gcal_event_set_date_start (dialog->event, start_date);
       gcal_event_set_date_end (dialog->event, end_date);
 
@@ -557,13 +570,10 @@ gcal_edit_dialog_set_time_format (GcalEditDialog *dialog,
 {
   g_return_if_fail (GCAL_IS_EDIT_DIALOG (dialog));
 
-  if (dialog->format_24h != use_24h_format)
-  {
-    dialog->format_24h = use_24h_format;
+  dialog->format_24h = use_24h_format;
 
-    gcal_time_selector_set_time_format (GCAL_TIME_SELECTOR (dialog->start_time_selector), dialog->format_24h);
-    gcal_time_selector_set_time_format (GCAL_TIME_SELECTOR (dialog->end_time_selector), dialog->format_24h);
-  }
+  gcal_time_selector_set_time_format (GCAL_TIME_SELECTOR (dialog->start_time_selector), dialog->format_24h);
+  gcal_time_selector_set_time_format (GCAL_TIME_SELECTOR (dialog->end_time_selector), dialog->format_24h);
 }
 
 void
@@ -621,6 +631,8 @@ gcal_edit_dialog_set_event (GcalEditDialog *dialog,
       fill_sources_menu (dialog);
 
       /* Load new event data */
+      gcal_edit_dialog_set_writable (dialog, gcal_manager_is_client_writable (dialog->manager, source));
+
       /* summary */
       summary = gcal_event_get_summary (event);
 
@@ -642,7 +654,11 @@ gcal_edit_dialog_set_event (GcalEditDialog *dialog,
       date_start = all_day ? g_date_time_ref (date_start) : g_date_time_to_local (date_start);
 
       date_end = gcal_event_get_date_end (event);
-      date_end = all_day ? g_date_time_ref (date_end) : g_date_time_to_local (date_end);
+      /*
+       * This is subtracting what has been added in gcal_edit_dialog_action_button_clicked ().
+       * See bug 769300.
+       */
+      date_end = all_day ? g_date_time_add_days (date_end, -1) : g_date_time_to_local (date_end);
 
       /* date */
       gcal_date_selector_set_date (GCAL_DATE_SELECTOR (dialog->start_date_selector), date_start);
@@ -653,9 +669,9 @@ gcal_edit_dialog_set_event (GcalEditDialog *dialog,
       gcal_time_selector_set_time (GCAL_TIME_SELECTOR (dialog->end_time_selector), date_end);
 
       /* all_day  */
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->all_day_check), all_day);
       gtk_widget_set_sensitive (dialog->start_time_selector, dialog->writable && !all_day);
       gtk_widget_set_sensitive (dialog->end_time_selector, dialog->writable && !all_day);
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->all_day_check), all_day);
 
       /* location */
       gtk_entry_set_text (GTK_ENTRY (dialog->location_entry), gcal_event_get_location (event));
@@ -665,8 +681,6 @@ gcal_edit_dialog_set_event (GcalEditDialog *dialog,
                                 gcal_event_get_description (event),
                                 -1);
 
-      gcal_edit_dialog_set_writable (dialog, gcal_manager_is_client_writable (dialog->manager, source));
-
       g_clear_pointer (&date_start, g_date_time_unref);
       g_clear_pointer (&date_end, g_date_time_unref);
 
