@@ -19,6 +19,8 @@
 #include "gcal-event.h"
 #include "gcal-utils.h"
 
+#define LIBICAL_TZID_PREFIX "/freeassociation.sourceforge.net/"
+
 struct _GcalEvent
 {
   GObject             parent;
@@ -95,7 +97,11 @@ get_timezone_from_ical (ECalComponentDateTime *comp)
       gchar *tzid;
       gint offset;
 
-      zone = icaltimezone_get_builtin_timezone (comp->tzid);
+      if (g_str_has_prefix (comp->tzid, LIBICAL_TZID_PREFIX))
+          zone = icaltimezone_get_builtin_timezone_from_tzid (comp->tzid);
+      else
+          zone = icaltimezone_get_builtin_timezone (comp->tzid);
+
       offset = icaltimezone_get_utc_offset (zone, comp->value, NULL);
       tzid = format_utc_offset (offset);
 
@@ -191,9 +197,9 @@ gcal_event_set_component_internal (GcalEvent     *self,
       ECalComponentDateTime end;
       icaltimetype date;
       GDateTime *date_start;
-      GTimeZone *zone_start;
+      GTimeZone *zone_start = NULL;
       GDateTime *date_end;
-      GTimeZone *zone_end;
+      GTimeZone *zone_end = NULL;
       gboolean start_is_all_day, end_is_all_day;
       gchar *description;
 
@@ -216,19 +222,29 @@ gcal_event_set_component_internal (GcalEvent     *self,
 
       /* Setup end date */
       e_cal_component_get_dtend (component, &end);
-      date = icaltime_normalize (*end.value);
-      zone_end = get_timezone_from_ical (&end);
-      date_end = g_date_time_new (zone_end,
-                                  date.year, date.month, date.day,
-                                  date.is_date ? 0 : date.hour,
-                                  date.is_date ? 0 : date.minute,
-                                  date.is_date ? 0 : date.second);
-      end_is_all_day = datetime_is_date (date_end);
 
-      self->dt_end = g_date_time_ref (date_end);
+      if(!end.value)
+        {
+            self->all_day = TRUE;
+        }
+      else
+        {
+          date = icaltime_normalize (*end.value);
+          zone_end = get_timezone_from_ical (&end);
+          date_end = g_date_time_new (zone_end,
+                                      date.year, date.month, date.day,
+                                      date.is_date ? 0 : date.hour,
+                                      date.is_date ? 0 : date.minute,
+                                      date.is_date ? 0 : date.second);
+          end_is_all_day = datetime_is_date (date_end);
 
-      /* Setup all day */
-      self->all_day = start_is_all_day && end_is_all_day;
+          self->dt_end = g_date_time_ref (date_end);
+
+          /* Setup all day */
+          self->all_day = start_is_all_day && end_is_all_day;
+
+          e_cal_component_free_datetime (&end);
+        }
 
       /* Setup description */
       description = get_desc_from_component (component, "\n\n");
@@ -244,6 +260,8 @@ gcal_event_set_component_internal (GcalEvent     *self,
       g_clear_pointer (&zone_start, g_time_zone_unref);
       g_clear_pointer (&zone_end, g_time_zone_unref);
       g_clear_pointer (&description, g_free);
+
+      e_cal_component_free_datetime (&start);
     }
 }
 
